import random

import pyglet
import cocos

from gamelib import tree, player, data, ninepatch

from euclid import *

class Background(cocos.layer.ScrollableLayer):
    def __init__(self):
        super(Background, self).__init__(0.618)
        self.image = pyglet.image.load(data.filepath('background.png'))

        l = [0]*8
        c = [255]*16
        t = [0]*8
        self.quad = self.batch.add(4, pyglet.gl.GL_QUADS,
            pyglet.graphics.TextureGroup(self.image.texture),
            ('v2f', l), ('c4B', c), ('t2f', t))

    def set_view(self, x, y, w, h):
        super(Background, self).set_view(x, y, w, h)
        x *= .618
        y *= .618
        w /= .618
        h /= .618
        self.quad.vertices[:] = [
            x, y,
            x+w, y,
            x+w, y+h,
            x, y+h,
        ]
        s1 = (x%128)/128.
        s2 = s1 + w/128.
        t1 = (y%128)/128.
        t2 = t1 + h/128.
        l = [
            s1, t1,
            s2, t1,
            s2, t2,
            s1, t2,
        ]
        self.quad.tex_coords[:] = l


class GameScene(cocos.scene.Scene):

    def __init__(self):
        super(GameScene, self).__init__()

        scroller = cocos.layer.ScrollingManager()
        #scroller.scale = .75
        self.add(scroller, z=0)

        background = Background()
        scroller.add(background, z=0)

        self.tree = tree.TreeLayer()
        scroller.add(self.tree, z=1)

        self.baddies = player.BaddiesLayer()
        scroller.add(self.baddies, z=2)

        self.player = player.PlayerLayer()
        scroller.add(self.player, z=3)

        self.schedule(self.tree.update)
        self.schedule(self.baddies.update)
        self.schedule(self.player.update)

        self.starting = True

        self.ui = cocos.layer.Layer()
        w, h = cocos.director.director.get_window_size()
        self.score = 0
        self.score_text = cocos.text.Label('Distance = 0.0m', (20, h-20),
            font_name='A Yummy Apology', font_size=32,
            color=(0x63, 0x3a, 0x81, 255),
            anchor_x='left', anchor_y='top')
        self.ui.add(self.score_text, z=1)
        self.score_bg = TextBGLayer(self.score_text)
        self.ui.add(self.score_bg, z=0)
        self.add(self.ui, z=4)

    def add_score(self, amount):
        self.score += amount
        self.score_text.element.text = 'Distance %.1fm'%self.score
        #self.score_bg.update()

    def game_over(self):
        from cocos.scenes.transitions import FadeTransition
        cocos.director.director.replace(FadeTransition(GameOver(self.score),
            color=(0xc6, 0xda, 0x94)))

class TextBGLayer(cocos.layer.Layer):
    def __init__(self, label):
        super(TextBGLayer, self).__init__()

        image = pyglet.resource.image('button-patch.png')
        self.patch = ninepatch.NinePatch(image)

        self.batch = pyglet.graphics.Batch()
        self.label = label
        self.l = None
        self.update()

    def update(self):
        if self.l is not None:
            self.l.delete()
        x = int(self.label.x)
        y = int(self.label.y) - self.label.element.content_height
        self.l = self.patch.draw_around(self.batch, x, y,
            self.label.element.content_width + 20,
            int(self.label.element.content_height-self.label.element.content_height*.25))

    def draw(self):
        pyglet.gl.glPushMatrix()
        self.transform()
        self.batch.draw()
        pyglet.gl.glPopMatrix()



class GameOver(cocos.scene.Scene):
    def __init__(self, score):
        super(GameOver, self).__init__()
        scroller = cocos.layer.ScrollingManager()
        self.add(scroller, z=0)
        scroller.add(Background())
        self.add(MessageLayer('Game Over (%.1fm)'%score), z=1)
        
class MessageLayer(cocos.layer.Layer):
    is_event_handler = True
    def __init__(self, message):
        super(MessageLayer, self).__init__()
        x,y = cocos.director.director.get_window_size()
        self.add(cocos.text.Label(message, (x/2, y/2),
            font_name='A Yummy Apology', font_size=64,
            color=(0x63, 0x3a, 0x81, 255),
            anchor_x='center', anchor_y='center'))
        
    def on_key_press(self, k, m):
        cocos.director.director.pop()
        return True

